import _defineProperty from '@babel/runtime/helpers/esm/defineProperty';
import _classCallCheck from '@babel/runtime/helpers/esm/classCallCheck';
import _createClass from '@babel/runtime/helpers/esm/createClass';
import _inherits from '@babel/runtime/helpers/esm/inherits';
import _createSuper from '@babel/runtime/helpers/esm/createSuper';
import { isEngine, isMobile, isHotkey, $, escape, isServer, CardActiveTrigger, CardType, Card, Parser, unescape, VIEW_CLASS_NAME, CARD_KEY, READY_CARD_KEY, decodeCardValue, CARD_VALUE_KEY, CARD_TYPE_KEY, Plugin } from '@hanfy/engine';
import _assertThisInitialized from '@babel/runtime/helpers/esm/assertThisInitialized';
import _get from '@babel/runtime/helpers/esm/get';
import _getPrototypeOf from '@babel/runtime/helpers/esm/getPrototypeOf';
import _classPrivateFieldLooseBase from '@babel/runtime/helpers/esm/classPrivateFieldLooseBase';
import _classPrivateFieldLooseKey from '@babel/runtime/helpers/esm/classPrivateFieldLooseKey';
import _objectSpread from '@babel/runtime/helpers/esm/objectSpread2';
import CodeMirror from 'codemirror';
import debounce from 'lodash/debounce';
import { defineComponent, resolveComponent, openBlock, createBlock, withCtx, createElementBlock, Fragment, renderList, createTextVNode, toDisplayString, createApp, toRaw } from 'vue';
import ASelect from 'ant-design-vue/es/select';
import 'ant-design-vue/es/select/style';
import 'codemirror/mode/shell/shell';
import 'codemirror/mode/clike/clike';
import 'codemirror/mode/css/css';
import 'codemirror/mode/dart/dart';
import 'codemirror/mode/diff/diff';
import 'codemirror/mode/dockerfile/dockerfile';
import 'codemirror/mode/erlang/erlang';
import 'codemirror/mode/go/go';
import 'codemirror/mode/groovy/groovy';
import 'codemirror/mode/htmlmixed/htmlmixed';
import 'codemirror/mode/http/http';
import 'codemirror/mode/javascript/javascript';
import 'codemirror/mode/jsx/jsx';
import 'codemirror/mode/cmake/cmake';
import 'codemirror/mode/markdown/markdown';
import 'codemirror/mode/octave/octave';
import 'codemirror/mode/nginx/nginx';
import 'codemirror/mode/pascal/pascal';
import 'codemirror/mode/perl/perl';
import 'codemirror/mode/php/php';
import 'codemirror/mode/powershell/powershell';
import 'codemirror/mode/protobuf/protobuf';
import 'codemirror/mode/python/python';
import 'codemirror/mode/r/r';
import 'codemirror/mode/ruby/ruby';
import 'codemirror/mode/rust/rust';
import 'codemirror/mode/sql/sql';
import 'codemirror/mode/swift/swift';
import 'codemirror/mode/vb/vb';
import 'codemirror/mode/velocity/velocity';
import 'codemirror/mode/xml/xml';
import 'codemirror/mode/yaml/yaml';
import 'codemirror/addon/scroll/simplescrollbars';
import 'codemirror/addon/scroll/simplescrollbars.css';

var qa = ['c', 'cpp', 'csharp', 'erlang', 'go', 'groovy', 'java', 'kotlin', 'makefile', 'objectivec', 'perl', 'python', 'rust', 'swift', 'vbnet'];
var defaultStyles = {
  header: 'color: blue;font-weight: bold;',
  quote: 'color: #090;',
  negative: 'color: #d44;',
  positive: 'color: #292;',
  strong: 'font-weight: bold;',
  em: 'font-style: italic;',
  link: 'text-decoration: underline;color: #00c;',
  strikethrough: 'text-decoration: line-through;',
  keyword: 'color: #d73a49;',
  atom: 'color: #905;',
  number: 'color: #005cc5;',
  def: 'color: #005cc5;',
  variable: '',
  'variable-2': 'color: #005cc5;',
  'variable-3': 'color: #22863a;',
  type: 'color: #22863a;',
  comment: 'color: #6a737d;',
  string: 'color: #690',
  'string-2': 'color: #690;',
  meta: 'color: #1f7f9a;',
  qualifier: 'color: #555;',
  builtin: 'color: #6f42c1;',
  bracket: 'color: #997;',
  tag: 'color: #22863a;',
  attribute: 'color: #6f42c1;',
  hr: 'color: #999;',
  error: 'color: #f00;',
  invalidchar: 'color: #f00;',
  operator: 'color: #d73a49;',
  property: 'color: #005cc5;'
};

var CodeBlockEditor = /*#__PURE__*/function () {
  function CodeBlockEditor(editor, options) {
    _classCallCheck(this, CodeBlockEditor);

    this.editor = void 0;
    this.options = void 0;
    this.styleMap = void 0;
    this.codeMirror = void 0;
    this.mode = 'plain';
    this.container = void 0;
    this.editor = editor;
    this.options = options;
    this.styleMap = _objectSpread(_objectSpread({}, defaultStyles), options.styleMap);
    this.container = options.container || $(this.renderTemplate());
  }

  _createClass(CodeBlockEditor, [{
    key: "renderTemplate",
    value: function renderTemplate() {
      return '<div class="data-codeblock-container"><div class="data-codeblock-content"></div></div>';
    }
  }, {
    key: "getConfig",
    value: function getConfig(value, mode) {
      var mirror = this.codeMirror;
      var editor = this.editor;
      var tabSize = mirror ? mirror.getOption('indentUnit') : qa.indexOf(mode || '') > -1 ? 4 : 2;
      var reg = value ? value.match(/^ {2,4}(?=[^\s])/gm) : null;

      if (reg) {
        tabSize = reg.reduce(function (val1, val2) {
          return Math.min(val1, val2.length);
        }, 1 / 0);
      }

      return {
        tabSize: tabSize,
        indentUnit: tabSize,
        scrollbarStyle: 'simple',
        readOnly: !isEngine(editor) || editor.readonly,
        viewportMargin: Infinity
      };
    }
  }, {
    key: "getSyntax",
    value: function getSyntax(mode) {
      return this.options.synatxMap[mode] || mode;
    }
  }, {
    key: "create",
    value: function create(mode, value, options) {
      var _this = this;

      this.mode = mode;
      var syntaxMode = this.getSyntax(mode);
      var mirror = CodeMirror(this.container.find('.data-codeblock-content').get(), _objectSpread(_objectSpread({
        value: value,
        mode: syntaxMode,
        lineNumbers: true,
        lineWrapping: false,
        autofocus: false,
        dragDrop: false
      }, this.getConfig(value, syntaxMode)), options));
      mirror.on('focus', function () {
        var onFocus = _this.options.onFocus;
        if (onFocus) onFocus();
      });
      mirror.on('blur', function () {
        var onBlur = _this.options.onBlur;
        if (onBlur) onBlur();
      });

      if (isMobile) {
        mirror.on('touchstart', function (_, event) {
          var onMouseDown = _this.options.onMouseDown;
          if (onMouseDown) onMouseDown(event);
        });
      } else {
        mirror.on('mousedown', function (_, event) {
          var onMouseDown = _this.options.onMouseDown;
          if (event.button === 2) event.stopPropagation();
          if (onMouseDown) onMouseDown(event);
        });
      }

      mirror.on('change', debounce(function () {
        if (!isEngine(_this.editor)) return;

        _this.save();
      }, 50));
      mirror.setOption('extraKeys', {
        Enter: function Enter(mirror) {
          var config = _this.getConfig(mirror.getValue());

          Object.keys(config).forEach(function (key) {
            return mirror.setOption(key, config[key]);
          });
          mirror.execCommand('newlineAndIndent');
        }
      });
      mirror.on('keydown', function (editor, event) {
        // 撤销和重做使用codemirror自带的操作
        if (isHotkey('mod+z', event) || isHotkey('mod+y', event) || isHotkey('mod+shift+z', event)) {
          event.stopPropagation();
        }

        var lineCount = editor.lineCount();

        var _editor$getCursor = editor.getCursor(),
            line = _editor$getCursor.line,
            ch = _editor$getCursor.ch;

        var _this$options = _this.options,
            onUpFocus = _this$options.onUpFocus,
            onDownFocus = _this$options.onDownFocus,
            onLeftFocus = _this$options.onLeftFocus,
            onRightFocus = _this$options.onRightFocus;
        var content = editor.getLine(line); // 在最后一行

        if (line === lineCount - 1 && ch === content.length) {
          // 按下下键
          if (isHotkey('down', event) || isHotkey('ctrl+n', event)) {
            if (onDownFocus) onDownFocus(event);
            return;
          } // 按下右键
          else if (isHotkey('right', event) || isHotkey('shift+right', event) || isHotkey('ctrl+e', event) || isHotkey('ctrl+f', event)) {
            if (onRightFocus) onRightFocus(event);
            return;
          }
        } // 在第一行按下上键


        if (line === 0 && ch === 0) {
          // 按下上键
          if (isHotkey('up', event) || isHotkey('ctrl+p', event)) {
            if (onUpFocus) onUpFocus(event);
          } // 按下左键
          else if (isHotkey('left', event) || isHotkey('shift+left', event) || isHotkey('ctrl+b', event) || isHotkey('ctrl+a', event)) {
            if (onLeftFocus) onLeftFocus(event);
          }
        }
      });
      this.container.on('mousedown', function (event) {
        if (!(mirror === null || mirror === void 0 ? void 0 : mirror.hasFocus())) {
          setTimeout(function () {
            mirror === null || mirror === void 0 ? void 0 : mirror.focus();
          }, 0);
        }
      });
      this.codeMirror = mirror;
      return this.codeMirror;
    }
  }, {
    key: "setAutoWrap",
    value: function setAutoWrap(value) {
      var _this$codeMirror;

      (_this$codeMirror = this.codeMirror) === null || _this$codeMirror === void 0 ? void 0 : _this$codeMirror.setOption('lineWrapping', value);
    }
  }, {
    key: "update",
    value: function update(mode, code) {
      this.mode = mode;
      var mirror = this.codeMirror;
      var editor = this.editor;

      if (code !== undefined) {
        mirror === null || mirror === void 0 ? void 0 : mirror.setValue(code);
      }

      mirror === null || mirror === void 0 ? void 0 : mirror.setOption('mode', this.getSyntax(mode));
      mirror === null || mirror === void 0 ? void 0 : mirror.setOption('readOnly', !isEngine(editor) || editor.readonly ? true : false);
      this.save();
    }
  }, {
    key: "render",
    value: function render(mode, value, options) {
      var root = this.container.find('.data-codeblock-content');
      mode = this.getSyntax(mode);
      var stage = $('<div style="font-family: monospace;font-size: 13px; line-height: 21px; color: #595959; direction: ltr; height: auto; overflow: hidden;background: transparent;"><pre style="color: rgb(89, 89, 89); margin: 0px; padding: 0px; background: none 0% 0% / auto repeat scroll padding-box border-box rgba(0, 0, 0, 0);" /></div>');
      root.append(stage);
      var pre = stage.find('pre')[0];
      this.runMode(value || '', mode, pre, _objectSpread(_objectSpread({}, this.getConfig(value, mode)), options));
    }
  }, {
    key: "save",
    value: function save() {
      var mirror = this.codeMirror;
      var editor = this.editor;
      if (!isEngine(editor) || !mirror) return; // 中文输入过程需要判断

      if (editor.change.isComposing()) {
        return;
      }

      var value = mirror.getValue();
      var onSave = this.options.onSave;
      if (onSave) onSave(this.mode, value);
    }
  }, {
    key: "focus",
    value: function focus() {
      var mirror = this.codeMirror;
      if (!mirror) return;
      mirror.focus();
    }
  }, {
    key: "select",
    value: function select() {
      var start = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;
      var mirror = this.codeMirror;
      if (!mirror) return;
      mirror.focus();

      if (!start) {
        var line = mirror.lineCount() - 1;
        var content = mirror.getLine(line);
        mirror.setSelection({
          line: line,
          ch: content.length
        });
      } else {
        mirror.setSelection({
          line: 0,
          ch: 0
        });
      }
    }
  }, {
    key: "toHtml",
    value: function toHtml(col, text) {
      var _this2 = this;

      var style = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
      var tabSize = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;
      tabSize = tabSize || CodeMirror.defaults.tabSize;
      var html = '';
      var content = ''; // replace tabs

      for (var pos = 0;;) {
        var idx = text.indexOf('\t', pos);

        if (idx === -1) {
          content += text.slice(pos);
          col += text.length - pos;
          break;
        } else {
          col += idx - pos;
          content += text.slice(pos, idx);
          var size = tabSize - col % tabSize;
          col += size;

          for (var i = 0; i < size; ++i) {
            content += ' ';
          }

          pos = idx + 1;
        }
      }

      if (style) {
        var styleStr = '';
        style.split(' ').forEach(function (cls) {
          var _this2$styleMap$cls;

          styleStr += (_this2$styleMap$cls = _this2.styleMap[cls]) !== null && _this2$styleMap$cls !== void 0 ? _this2$styleMap$cls : '';
        });
        var spanElement = "<span ".concat(styleStr ? "style=\"".concat(styleStr, "\"") : '', ">").concat(escape(content), "</span>");
        html += spanElement;
      } else {
        html += content;
      }

      return html;
    }
    /**
     * 代码来自 runmode addon
     * 支持行号需要考虑复制粘贴问题
     *
     * runmode 本身不支持行号，见 https://github.com/codemirror/CodeMirror/issues/3364
     * 可参考的解法  https://stackoverflow.com/questions/14237361/use-codemirror-for-standalone-syntax-highlighting-with-line-numbers
     *
     * ref:
     * - https://codemirror.net/doc/manual.html#addons
     * - https://codemirror.net/addon/runmode/runmode.js
     */

  }, {
    key: "runMode",
    value: function runMode(string, modespec, callback, options) {
      var mode = CodeMirror.getMode(CodeMirror.defaults, modespec);
      var tabSize = options && options.tabSize || CodeMirror.defaults.tabSize;
      var node = callback;
      var col = 0;
      node.innerHTML = '';
      var html = '';
      var lines = CodeMirror.splitLines(string);
      var state = options && options.state || CodeMirror.startState(mode);

      for (var i = 0, e = lines.length; i < e; ++i) {
        if (i) {
          html += '<br />';
          col = 0;
        }

        var stream = new CodeMirror.StringStream(lines[i]);
        if (!stream.string && mode.blankLine) mode.blankLine(state);

        while (!stream.eol()) {
          var style = mode.token ? mode.token(stream, state) : '';
          html += this.toHtml(col, stream.current(), style || '', tabSize);
          stream.start = stream.pos;
        }
      }

      node.innerHTML = html;
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.container.remove();
    }
  }]);

  return CodeBlockEditor;
}();

const ASelectOption = ASelect.Option;
var script = defineComponent({
    name: "am-codeblock-select",
    components: {
        ASelect,
        ASelectOption
    },
    props: {
        modeDatas: Array,
        defaultValue: String,
        getContainer: Function,
        onSelect: Function
    },
    methods: {
        filter(input, option) {
            input = input.toLowerCase();
            const key = option.key || '';
            let name = option.name || '';
            name = name.toLowerCase();
            return key.includes(input) || name.includes(input);
        }
    }
});

function render(_ctx, _cache, $props, $setup, $data, $options) {
  const _component_a_select_option = resolveComponent("a-select-option");
  const _component_a_select = resolveComponent("a-select");

  return (openBlock(), createBlock(_component_a_select, {
    "show-search": true,
    size: "small",
    bordered: false,
    style: {"min-width":"128px"},
    "default-value": _ctx.defaultValue,
    "get-popup-container": _ctx.getContainer,
    onSelect: _ctx.onSelect,
    "filter-option": _ctx.filter
  }, {
    default: withCtx(() => [
      (openBlock(true), createElementBlock(Fragment, null, renderList(_ctx.modeDatas, (item) => {
        return (openBlock(), createBlock(_component_a_select_option, {
          name: item.name,
          value: item.value,
          key: item.value
        }, {
          default: withCtx(() => [
            createTextVNode(toDisplayString(item.name), 1 /* TEXT */)
          ]),
          _: 2 /* DYNAMIC */
        }, 1032 /* PROPS, DYNAMIC_SLOTS */, ["name", "value"]))
      }), 128 /* KEYED_FRAGMENT */))
    ]),
    _: 1 /* STABLE */
  }, 8 /* PROPS */, ["default-value", "get-popup-container", "onSelect", "filter-option"]))
}

script.render = render;
script.__file = "src/component/select/component.vue";

var renderSelect = (function (container, modeDatas, defaultValue, onSelect) {
  var vm = createApp(script, {
    modeDatas: modeDatas,
    defaultValue: defaultValue,
    getContainer: container ? function () {
      return container;
    } : undefined,
    onSelect: onSelect
  });
  vm.mount(container);
  return vm;
});

var datas = [{
  value: 'plain',
  syntax: 'simplemode',
  name: 'Plain Text'
}, {
  value: 'bash',
  syntax: 'shell',
  name: 'Bash'
}, {
  value: 'basic',
  syntax: 'vbscript',
  name: 'Basic'
}, {
  value: 'c',
  syntax: 'text/x-csrc',
  name: 'C'
}, {
  value: 'cpp',
  syntax: 'text/x-c++src',
  alias: ['c++'],
  name: 'C++'
}, {
  value: 'csharp',
  syntax: 'text/x-csharp',
  alias: ['c#'],
  name: 'C#'
}, {
  value: 'css',
  syntax: 'css',
  name: 'CSS'
}, {
  value: 'dart',
  syntax: 'dart',
  name: 'Dart'
}, {
  value: 'diff',
  syntax: 'diff',
  name: 'Diff'
}, {
  value: 'dockerfile',
  syntax: 'dockerfile',
  name: 'Dockerfile'
}, {
  value: 'erlang',
  syntax: 'erlang',
  name: 'Erlang'
}, {
  value: 'git',
  syntax: 'shell',
  name: 'Git'
}, {
  value: 'go',
  syntax: 'go',
  alias: ['golang'],
  name: 'Go'
}, {
  value: 'graphql',
  syntax: 'simplemode',
  name: 'GraphQL'
}, {
  value: 'groovy',
  syntax: 'groovy',
  name: 'Groovy'
}, {
  value: 'html',
  syntax: 'htmlmixed',
  name: 'HTML',
  alias: ['html5']
}, {
  value: 'http',
  syntax: 'http',
  name: 'HTTP'
}, {
  value: 'java',
  syntax: 'text/x-java',
  name: 'Java'
}, {
  value: 'javascript',
  syntax: 'text/javascript',
  name: 'JavaScript',
  alias: ['js']
}, {
  value: 'json',
  syntax: 'application/json',
  name: 'JSON'
}, {
  value: 'jsx',
  syntax: 'jsx',
  name: 'JSX'
}, {
  value: 'katex',
  syntax: 'simplemode',
  name: 'KaTeX'
}, {
  value: 'kotlin',
  syntax: 'text/x-kotlin',
  name: 'Kotlin'
}, {
  value: 'less',
  syntax: 'css',
  name: 'Less'
}, {
  value: 'makefile',
  syntax: 'cmake',
  name: 'Makefile'
}, {
  value: 'markdown',
  syntax: 'markdown',
  name: 'Markdown'
}, {
  value: 'matlab',
  syntax: 'octave',
  name: 'MATLAB'
}, {
  value: 'nginx',
  syntax: 'nginx',
  name: 'Nginx'
}, {
  value: 'objectivec',
  syntax: 'text/x-objectivec',
  name: 'Objective-C'
}, {
  value: 'pascal',
  syntax: 'pascal',
  name: 'Pascal'
}, {
  value: 'perl',
  syntax: 'perl',
  name: 'Perl'
}, {
  value: 'php',
  syntax: 'php',
  name: 'PHP'
}, {
  value: 'powershell',
  syntax: 'powershell',
  name: 'PowerShell'
}, {
  value: 'protobuf',
  syntax: 'protobuf',
  name: 'Protobuf'
}, {
  value: 'python',
  syntax: 'python',
  name: 'Python',
  alias: ['py']
}, {
  value: 'r',
  syntax: 'r',
  name: 'R'
}, {
  value: 'ruby',
  syntax: 'ruby',
  name: 'Ruby'
}, {
  value: 'rust',
  syntax: 'rust',
  name: 'Rust'
}, {
  value: 'scala',
  syntax: 'text/x-scala',
  name: 'Scala'
}, {
  value: 'shell',
  syntax: 'shell',
  name: 'Shell'
}, {
  value: 'sql',
  syntax: 'text/x-sql',
  name: 'SQL'
}, {
  value: 'swift',
  syntax: 'swift',
  name: 'Swift'
}, {
  value: 'typescript',
  syntax: 'text/typescript',
  name: 'TypeScript',
  alias: ['ts']
}, {
  value: 'vbnet',
  syntax: 'vb',
  name: 'VB.net'
}, {
  value: 'velocity',
  syntax: 'velocity',
  name: 'Velocity'
}, {
  value: 'xml',
  syntax: 'xml',
  name: 'XML'
}, {
  value: 'yaml',
  syntax: 'yaml',
  name: 'YAML'
}];

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".data-codeblock-container {\n     position: relative;\n     padding: 0;\n     overflow: visible;\n     border: 1px solid #e8e8e8;\n     border-radius: 2px 2px;\n     text-indent: 0;\n }\n \n .data-codeblock-container .am-embed-toolbar-item {\n     float: right;\n     line-height: 32px;\n }\n \n .data-codeblock-content {\n      background: #f9f9f9;\n      overflow: auto;\n }\n \n .am-engine .CodeMirror,\n .am-engine-view .CodeMirror {\n      font-family: monospace;\n      font-size: 13px;\n      line-height: 21px;\n      color: #595959;\n      direction: ltr;\n      height: auto;\n      overflow: hidden;\n      background: transparent;\n }\n \n .am-engine .CodeMirror-lines,\n .am-engine-view .CodeMirror-lines {\n      padding: 8px 0;\n }\n \n .am-engine .CodeMirror-scrollbar-filler,\n .am-engine-view .CodeMirror-scrollbar-filler,\n .am-engine .CodeMirror-gutter-filler,\n .am-engine-view .CodeMirror-gutter-filler {\n      background-color: white;\n     \n     /* The little square between H and V scrollbars */\n     \n }\n \n .am-engine .CodeMirror-gutters,\n .am-engine-view .CodeMirror-gutters {\n      border: 0;\n      white-space: nowrap;\n      padding: 0 8px;\n      background-color: unset;\n     \n }\n \n .am-engine .CodeMirror-linenumber,\n .am-engine-view .CodeMirror-linenumber {\n      padding: 0;\n      min-width: 20px;\n      text-align: right;\n      color: #BFBFBF;\n      white-space: nowrap;\n     \n }\n \n .am-engine .CodeMirror-guttermarker,\n .am-engine-view .CodeMirror-guttermarker {\n      color: black;\n     \n }\n \n .am-engine .CodeMirror-guttermarker-subtle,\n .am-engine-view .CodeMirror-guttermarker-subtle {\n      color: #999;\n     \n }\n \n .am-engine .CodeMirror-cursor,\n .am-engine-view .CodeMirror-cursor {\n      border-left: 1px solid black;\n      border-right: none;\n      width: 0;\n     \n }\n \n .am-engine .CodeMirror div.CodeMirror-secondarycursor,\n .am-engine-view .CodeMirror div.CodeMirror-secondarycursor {\n      border-left: 1px solid silver;\n     \n }\n \n .am-engine .cm-fat-cursor .CodeMirror-cursor,\n .am-engine-view .cm-fat-cursor .CodeMirror-cursor {\n      width: auto;\n      border: 0 !important;\n      background: #7e7;\n     \n }\n \n .am-engine .cm-fat-cursor div.CodeMirror-cursors,\n .am-engine-view .cm-fat-cursor div.CodeMirror-cursors {\n      z-index: 1;\n     \n }\n \n .am-engine .cm-fat-cursor-mark,\n .am-engine-view .cm-fat-cursor-mark {\n      background-color: rgba(20, 255, 20, 0.5);\n      -webkit-animation: blink 1.06s steps(1) infinite;\n      animation: blink 1.06s steps(1) infinite;\n     \n }\n \n .am-engine .cm-animate-fat-cursor,\n .am-engine-view .cm-animate-fat-cursor {\n      width: auto;\n      border: 0;\n      -webkit-animation: blink 1.06s steps(1) infinite;\n      animation: blink 1.06s steps(1) infinite;\n      background-color: #7e7;\n     \n }\n \n @-webkit-keyframes blink {\n      50% {\n          background-color: transparent;\n         \n     }\n \n     \n }\n \n @keyframes blink {\n      50% {\n          background-color: transparent;\n         \n     }\n \n     \n }\n \n .am-engine .cm-tab,\n .am-engine-view .cm-tab {\n      display: inline-block;\n      text-decoration: inherit;\n     \n }\n \n .am-engine .CodeMirror-rulers,\n .am-engine-view .CodeMirror-rulers {\n      position: absolute;\n      left: 0;\n      right: 0;\n      top: -50px;\n      bottom: -20px;\n      overflow: hidden;\n     \n }\n \n .am-engine .CodeMirror-ruler,\n .am-engine-view .CodeMirror-ruler {\n      border-left: 1px solid #ccc;\n      top: 0;\n      bottom: 0;\n      position: absolute;\n     \n }\n \n .am-engine .cm-s-default .cm-header,\n .am-engine-view .cm-s-default .cm-header {\n      color: blue;\n     \n }\n \n .am-engine .cm-s-default .cm-quote,\n .am-engine-view .cm-s-default .cm-quote {\n      color: #090;\n     \n }\n \n .am-engine .cm-negative,\n .am-engine-view .cm-negative {\n      color: #d44;\n     \n }\n \n .am-engine .cm-positive,\n .am-engine-view .cm-positive {\n      color: #292;\n     \n }\n \n .am-engine .cm-header,\n .am-engine-view .cm-header,\n .am-engine .cm-strong,\n .am-engine-view .cm-strong {\n      font-weight: bold;\n     \n }\n \n .am-engine .cm-em,\n .am-engine-view .cm-em {\n      font-style: italic;\n     \n }\n \n .am-engine .cm-link,\n .am-engine-view .cm-link {\n      text-decoration: underline;\n     \n }\n \n .am-engine .cm-strikethrough,\n .am-engine-view .cm-strikethrough {\n      text-decoration: line-through;\n     \n }\n \n .am-engine .cm-s-default .cm-keyword,\n .am-engine-view .cm-s-default .cm-keyword {\n      color: #d73a49;\n     \n }\n \n .am-engine .cm-s-default .cm-atom,\n .am-engine-view .cm-s-default .cm-atom {\n      color: #905;\n     \n }\n \n .am-engine .cm-s-default .cm-number,\n .am-engine-view .cm-s-default .cm-number {\n      color: #005cc5;\n     \n }\n \n .am-engine .cm-s-default .cm-def,\n .am-engine-view .cm-s-default .cm-def {\n      color: #005cc5;\n     \n }\n \n .am-engine .cm-s-default .cm-variable-2,\n .am-engine-view .cm-s-default .cm-variable-2 {\n      color: #005cc5;\n     \n }\n \n .am-engine .cm-s-default .cm-variable-3,\n .am-engine-view .cm-s-default .cm-variable-3,\n .am-engine .cm-s-default .cm-type,\n .am-engine-view .cm-s-default .cm-type {\n      color: #22863a;\n     \n }\n \n .am-engine .cm-s-default .cm-comment,\n .am-engine-view .cm-s-default .cm-comment {\n      color: #6a737d;\n     \n }\n \n .am-engine .cm-s-default .cm-string,\n .am-engine-view .cm-s-default .cm-string {\n      color: #690;\n     \n }\n \n .am-engine .cm-s-default .cm-string-2,\n .am-engine-view .cm-s-default .cm-string-2 {\n      color: #690;\n     \n }\n \n .am-engine .cm-s-default .cm-meta,\n .am-engine-view .cm-s-default .cm-meta {\n      color: #1f7f9a;\n     \n }\n \n .am-engine .cm-s-default .cm-qualifier,\n .am-engine-view .cm-s-default .cm-qualifier {\n      color: #555;\n     \n }\n \n .am-engine .cm-s-default .cm-builtin,\n .am-engine-view .cm-s-default .cm-builtin {\n      color: #6f42c1;\n     \n }\n \n .am-engine .cm-s-default .cm-bracket,\n .am-engine-view .cm-s-default .cm-bracket {\n      color: #997;\n     \n }\n \n .am-engine .cm-s-default .cm-tag,\n .am-engine-view .cm-s-default .cm-tag {\n      color: #22863a;\n     \n }\n \n .am-engine .cm-s-default .cm-attribute,\n .am-engine-view .cm-s-default .cm-attribute {\n      color: #6f42c1;\n     \n }\n \n .am-engine .cm-s-default .cm-hr,\n .am-engine-view .cm-s-default .cm-hr {\n      color: #999;\n     \n }\n \n .am-engine .cm-s-default .cm-link,\n .am-engine-view .cm-s-default .cm-link {\n      color: #00c;\n     \n }\n \n .am-engine .cm-s-default .cm-error,\n .am-engine-view .cm-s-default .cm-error {\n      color: #f00;\n     \n }\n \n .am-engine .cm-invalidchar,\n .am-engine-view .cm-invalidchar {\n      color: #f00;\n     \n }\n \n .am-engine .cm-s-default .cm-operator,\n .am-engine-view .cm-s-default .cm-operator {\n      color: #d73a49;\n     \n }\n \n .am-engine .cm-s-default .cm-property,\n .am-engine-view .cm-s-default .cm-property {\n      color: #005cc5;\n     \n }\n \n .am-engine .CodeMirror-composing,\n .am-engine-view .CodeMirror-composing {\n      border-bottom: 2px solid;\n     \n }\n \n .am-engine div.CodeMirror span.CodeMirror-matchingbracket,\n .am-engine-view div.CodeMirror span.CodeMirror-matchingbracket {\n      color: #0b0;\n     \n }\n \n .am-engine div.CodeMirror span.CodeMirror-nonmatchingbracket,\n .am-engine-view div.CodeMirror span.CodeMirror-nonmatchingbracket {\n      color: #a22;\n     \n }\n \n .am-engine .CodeMirror-matchingtag,\n .am-engine-view .CodeMirror-matchingtag {\n      background: rgba(255, 150, 0, 0.3);\n     \n }\n \n .am-engine .CodeMirror-activeline-background,\n .am-engine-view .CodeMirror-activeline-background {\n      background: transparent;\n     \n }\n \n .am-engine .CodeMirror-scroll,\n .am-engine-view .CodeMirror-scroll {\n      overflow: scroll !important;\n     \n     /* Things will break if this is overridden */\n     \n     /* 30px is the magic margin used to hide the element's real scrollbars */\n     \n     /* See overflow: hidden in .CodeMirror */\n      margin-bottom: -50px;\n      margin-right: -50px;\n      padding-bottom: 50px;\n      outline: none;\n     \n     /* Prevent dragging from highlighting the element */\n      position: relative;\n      z-index: 0;\n     \n }\n \n .am-engine .CodeMirror-sizer,\n .am-engine-view .CodeMirror-sizer {\n      position: relative;\n      border-right: 50px solid transparent;\n      min-height: auto !important;\n     \n }\n \n .am-engine .CodeMirror-vscrollbar,\n .am-engine-view .CodeMirror-vscrollbar,\n .am-engine .CodeMirror-hscrollbar,\n .am-engine-view .CodeMirror-hscrollbar,\n .am-engine .CodeMirror-scrollbar-filler,\n .am-engine-view .CodeMirror-scrollbar-filler,\n .am-engine .CodeMirror-gutter-filler,\n .am-engine-view .CodeMirror-gutter-filler {\n      position: absolute;\n      z-index: 6;\n      display: none;\n     \n }\n \n .am-engine .CodeMirror-vscrollbar,\n .am-engine-view .CodeMirror-vscrollbar {\n      right: 0;\n      top: 0;\n      overflow-x: hidden;\n      overflow-y: scroll;\n      display: none;\n     \n }\n \n .am-engine .CodeMirror-hscrollbar,\n .am-engine-view .CodeMirror-hscrollbar {\n      bottom: 0;\n      left: 0;\n      overflow-y: hidden;\n      overflow-x: scroll;\n      display: none;\n     \n }\n \n .am-engine .CodeMirror-scrollbar-filler,\n .am-engine-view .CodeMirror-scrollbar-filler {\n      right: 0;\n      bottom: 0;\n     \n }\n \n .am-engine .CodeMirror-gutter-filler,\n .am-engine-view .CodeMirror-gutter-filler {\n      left: 0;\n      bottom: 0;\n     \n }\n \n .am-engine .CodeMirror-gutters,\n .am-engine-view .CodeMirror-gutters {\n      position: absolute;\n      left: 0;\n      top: 0;\n      min-height: 100%;\n      background: #f9f9f9;\n      z-index: 3;\n }\n \n .am-engine .CodeMirror-gutter,\n .am-engine-view .CodeMirror-gutter {\n      white-space: normal;\n      height: 100%;\n      display: inline-block;\n      vertical-align: top;\n      margin-bottom: -30px;\n     \n }\n \n .am-engine .CodeMirror-gutter-wrapper,\n .am-engine-view .CodeMirror-gutter-wrapper {\n      position: absolute;\n      z-index: 4;\n      background: none !important;\n      border: none !important;\n     \n }\n \n .am-engine .CodeMirror-gutter-background,\n .am-engine-view .CodeMirror-gutter-background {\n      position: absolute;\n      top: 0;\n      bottom: 0;\n      z-index: 4;\n     \n }\n \n .am-engine .CodeMirror-gutter-elt,\n .am-engine-view .CodeMirror-gutter-elt {\n      position: absolute;\n      cursor: default;\n      z-index: 4;\n     \n }\n \n .am-engine .CodeMirror-gutter-wrapper ::-moz-selection,\n .am-engine-view .CodeMirror-gutter-wrapper ::-moz-selection {\n      background-color: transparent;\n     \n }\n \n .am-engine .CodeMirror-gutter-wrapper ::selection,\n .am-engine-view .CodeMirror-gutter-wrapper ::selection {\n      background-color: transparent;\n     \n }\n \n .am-engine .CodeMirror-gutter-wrapper ::-moz-selection,\n .am-engine-view .CodeMirror-gutter-wrapper ::-moz-selection {\n      background-color: transparent;\n     \n }\n \n .am-engine .CodeMirror-lines,\n .am-engine-view .CodeMirror-lines {\n      cursor: text;\n      min-height: 40px;\n     \n     /* prevents collapsing before first draw */\n     \n }\n \n .am-engine .CodeMirror pre,\n .am-engine-view .CodeMirror pre {\n     \n     /* Reset some styles that the rest of the page might have set */\n      border-radius: 0;\n      border-width: 0;\n      background: transparent;\n      font-family: inherit;\n      font-size: inherit;\n      margin: 0;\n      white-space: pre;\n      word-wrap: normal;\n      line-height: inherit;\n      color: inherit;\n      z-index: 2;\n      position: relative;\n      overflow: visible;\n      -webkit-tap-highlight-color: transparent;\n      font-variant-ligatures: contextual;\n     \n }\n \n .am-engine .CodeMirror-wrap pre,\n .am-engine-view .CodeMirror-wrap pre {\n      word-wrap: break-word;\n      white-space: pre-wrap;\n      word-break: normal;\n     \n }\n \n .am-engine .CodeMirror-linebackground,\n .am-engine-view .CodeMirror-linebackground {\n      position: absolute;\n      left: 0;\n      right: 0;\n      top: 0;\n      bottom: 0;\n      z-index: 0;\n     \n }\n \n .am-engine .CodeMirror-linewidget,\n .am-engine-view .CodeMirror-linewidget {\n      position: relative;\n      z-index: 2;\n      padding: 0.1px;\n     \n     /* Force widget margins to stay inside of the container */\n     \n }\n \n .am-engine .CodeMirror-rtl pre,\n .am-engine-view .CodeMirror-rtl pre {\n      direction: rtl;\n     \n }\n \n .am-engine .CodeMirror-code,\n .am-engine-view .CodeMirror-code {\n      outline: none;\n     \n }\n \n .am-engine .CodeMirror-scroll,\n .am-engine-view .CodeMirror-scroll,\n .am-engine .CodeMirror-sizer,\n .am-engine-view .CodeMirror-sizer,\n .am-engine .CodeMirror-gutter,\n .am-engine-view .CodeMirror-gutter,\n .am-engine .CodeMirror-gutters,\n .am-engine-view .CodeMirror-gutters,\n .am-engine .CodeMirror-linenumber,\n .am-engine-view .CodeMirror-linenumber {\n      box-sizing: content-box;\n     \n }\n \n .am-engine .CodeMirror-simplescroll-horizontal, .am-engine-view .CodeMirror-simplescroll-horizontal {\n      bottom: 4px;\n      cursor: pointer;\n }\n \n .am-engine .CodeMirror-measure,\n .am-engine-view .CodeMirror-measure {\n      position: absolute;\n      width: 100%;\n      height: 0;\n      overflow: hidden;\n      visibility: hidden;\n     \n }\n \n .am-engine .CodeMirror-cursor,\n .am-engine-view .CodeMirror-cursor {\n      position: absolute;\n      pointer-events: none;\n     \n }\n \n .am-engine .CodeMirror-measure pre,\n .am-engine-view .CodeMirror-measure pre {\n      position: static;\n     \n }\n \n .am-engine div.CodeMirror-cursors,\n .am-engine-view div.CodeMirror-cursors {\n      visibility: hidden;\n      position: relative;\n      z-index: 3;\n     \n }\n \n .am-engine div.CodeMirror-dragcursors,\n .am-engine-view div.CodeMirror-dragcursors {\n      visibility: visible;\n     \n }\n \n .am-engine .CodeMirror-focused div.CodeMirror-cursors,\n .am-engine-view .CodeMirror-focused div.CodeMirror-cursors {\n      visibility: visible;\n     \n }\n \n .am-engine .CodeMirror-selected,\n .am-engine-view .CodeMirror-selected {\n      background: #e8e8e8;\n     \n }\n \n .am-engine .CodeMirror-focused .CodeMirror-selected,\n .am-engine-view .CodeMirror-focused .CodeMirror-selected {\n      background: #b3d7fd;\n     \n }\n \n .am-engine .CodeMirror-crosshair,\n .am-engine-view .CodeMirror-crosshair {\n      cursor: crosshair;\n     \n }\n \n .am-engine .CodeMirror-line::-moz-selection,\n .am-engine-view .CodeMirror-line::-moz-selection,\n .am-engine .CodeMirror-line>span::-moz-selection,\n .am-engine-view .CodeMirror-line>span::-moz-selection,\n .am-engine .CodeMirror-line>span>span::-moz-selection,\n .am-engine-view .CodeMirror-line>span>span::-moz-selection {\n      background: #b3d7fd;\n     \n }\n \n .am-engine .CodeMirror-line::selection,\n .am-engine-view .CodeMirror-line::selection,\n .am-engine .CodeMirror-line>span::selection,\n .am-engine-view .CodeMirror-line>span::selection,\n .am-engine .CodeMirror-line>span>span::selection,\n .am-engine-view .CodeMirror-line>span>span::selection {\n      background: #b3d7fd;\n     \n }\n \n .am-engine .CodeMirror-line::-moz-selection,\n .am-engine-view .CodeMirror-line::-moz-selection,\n .am-engine .CodeMirror-line>span::-moz-selection,\n .am-engine-view .CodeMirror-line>span::-moz-selection,\n .am-engine .CodeMirror-line>span>span::-moz-selection,\n .am-engine-view .CodeMirror-line>span>span::-moz-selection {\n      background: #b3d7fd;\n     \n }\n \n .am-engine .cm-searching,\n .am-engine-view .cm-searching {\n      background-color: #ffa;\n      background-color: rgba(255, 255, 0, 0.4);\n     \n }\n \n .am-engine .cm-force-border,\n .am-engine-view .cm-force-border {\n      padding-right: 0.1px;\n     \n }\n \n @media print {\n \n     /* .am-engine,\n      .am-engine-view { */\n         \n         /* Hide the cursor when printing */\n         \n     /* } */\n \n      .am-engine .CodeMirror div.CodeMirror-cursors,\n      .am-engine-view .CodeMirror div.CodeMirror-cursors {\n          visibility: hidden;\n         \n     }\n \n     \n }\n \n .am-engine .cm-tab-wrap-hack:after,\n .am-engine-view .cm-tab-wrap-hack:after {\n      content: '';\n     \n }\n \n .am-engine span.CodeMirror-selectedtext,\n .am-engine-view span.CodeMirror-selectedtext {\n      background: none;\n     \n }\n \n .am-engine-view .data-codeblock-container {\n      margin: 5px 0;\n     \n }\n \n .am-content-editor .am-engine .data-codeblock-container .CodeMirror-lines {\n     min-height: 40px;\n }";
styleInject(css_248z);

var _modeNameMap = /*#__PURE__*/_classPrivateFieldLooseKey("modeNameMap");

var _modeSynatxMap = /*#__PURE__*/_classPrivateFieldLooseKey("modeSynatxMap");

var _viewAutoWrap = /*#__PURE__*/_classPrivateFieldLooseKey("viewAutoWrap");

var CodeBlcok = /*#__PURE__*/function (_Card) {
  _inherits(CodeBlcok, _Card);

  var _super = _createSuper(CodeBlcok);

  function CodeBlcok() {
    var _this;

    _classCallCheck(this, CodeBlcok);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.mirror = void 0;

    _this.resize = function () {
      var _this$codeEditor;

      return (_this$codeEditor = _this.codeEditor) === null || _this$codeEditor === void 0 ? void 0 : _this$codeEditor.container.find('.data-codeblock-content');
    };

    _this.codeEditor = void 0;
    Object.defineProperty(_assertThisInitialized(_this), _modeNameMap, {
      writable: true,
      value: {}
    });
    Object.defineProperty(_assertThisInitialized(_this), _modeSynatxMap, {
      writable: true,
      value: {}
    });
    Object.defineProperty(_assertThisInitialized(_this), _viewAutoWrap, {
      writable: true,
      value: undefined
    });
    return _this;
  }

  _createClass(CodeBlcok, [{
    key: "init",
    value: function init() {
      var _this2 = this;

      if (isServer) return;

      _get(_getPrototypeOf(CodeBlcok.prototype), "init", this).call(this);

      if (this.codeEditor) return;
      datas.forEach(function (item) {
        _classPrivateFieldLooseBase(_this2, _modeNameMap)[_modeNameMap][item.value] = item.name;
        _classPrivateFieldLooseBase(_this2, _modeSynatxMap)[_modeSynatxMap][item.value] = item.syntax;

        if (item.alias) {
          item.alias.forEach(function (name) {
            _classPrivateFieldLooseBase(_this2, _modeNameMap)[_modeNameMap][name] = item.name;
            _classPrivateFieldLooseBase(_this2, _modeSynatxMap)[_modeSynatxMap][name] = item.syntax;
          });
        }
      });
      var editor = this.editor;
      this.codeEditor = new CodeBlockEditor(editor, {
        synatxMap: _classPrivateFieldLooseBase(this, _modeSynatxMap)[_modeSynatxMap],
        onSave: function onSave(mode, value) {
          var oldValue = _this2.getValue();

          if (mode === (oldValue === null || oldValue === void 0 ? void 0 : oldValue.mode) && value === oldValue.code) return;

          _this2.setValue({
            mode: mode,
            code: value
          });
        },
        onMouseDown: function onMouseDown(event) {
          if (!_this2.activated) setTimeout(function () {
            editor.card.activate(_this2.root, CardActiveTrigger.MOUSE_DOWN);
          }, 10);
        },
        onUpFocus: function onUpFocus(event) {
          var _this2$toolbarModel;

          if (!isEngine(editor)) return;
          event.preventDefault();
          var change = editor.change,
              card = editor.card;
          var range = change.range.get().cloneRange();

          var prev = _this2.root.prev();

          var cardComponent = prev ? card.find(prev) : undefined;

          if (cardComponent === null || cardComponent === void 0 ? void 0 : cardComponent.onSelectUp) {
            cardComponent.onSelectUp(event);
          } else if (prev) {
            card.focusPrevBlock(_this2, range, false);
            change.range.select(range);
          } else {
            _this2.focus(range, true);

            change.range.select(range);
            return;
          }

          _this2.activate(false);

          (_this2$toolbarModel = _this2.toolbarModel) === null || _this2$toolbarModel === void 0 ? void 0 : _this2$toolbarModel.hide();
        },
        onDownFocus: function onDownFocus(event) {
          var _this2$toolbarModel2;

          if (!isEngine(editor)) return;
          event.preventDefault();
          var change = editor.change,
              card = editor.card;
          var range = change.range.get().cloneRange();

          var next = _this2.root.next();

          var cardComponent = next ? card.find(next) : undefined;

          if (cardComponent === null || cardComponent === void 0 ? void 0 : cardComponent.onSelectDown) {
            cardComponent.onSelectDown(event);
          } else if (next) {
            card.focusNextBlock(_this2, range, false);
            change.range.select(range);
          } else {
            _this2.focus(range, false);

            change.range.select(range);
            return;
          }

          _this2.activate(false);

          (_this2$toolbarModel2 = _this2.toolbarModel) === null || _this2$toolbarModel2 === void 0 ? void 0 : _this2$toolbarModel2.hide();
        },
        onLeftFocus: function onLeftFocus(event) {
          var _this2$toolbarModel3;

          if (!isEngine(editor)) return;
          event.preventDefault();
          var change = editor.change;
          var range = change.range.get().cloneRange();

          _this2.focus(range, true);

          change.range.select(range);

          _this2.activate(false);

          (_this2$toolbarModel3 = _this2.toolbarModel) === null || _this2$toolbarModel3 === void 0 ? void 0 : _this2$toolbarModel3.hide();
        },
        onRightFocus: function onRightFocus(event) {
          var _this2$toolbarModel4;

          if (!isEngine(editor)) return;
          event.preventDefault();
          var change = editor.change;
          var range = change.range.get().cloneRange();

          _this2.focus(range, false);

          change.range.select(range);

          _this2.activate(false);

          (_this2$toolbarModel4 = _this2.toolbarModel) === null || _this2$toolbarModel4 === void 0 ? void 0 : _this2$toolbarModel4.hide();
        }
      });
    }
  }, {
    key: "toolbar",
    value: function toolbar() {
      var _this3 = this,
          _editor$plugin$findPl;

      var editor = this.editor;

      var getItems = function getItems() {
        if (_this3.loading) return [];

        if (!isEngine(editor) || editor.readonly) {
          return [{
            key: 'copy',
            type: 'copy'
          }, {
            key: 'autoWrap',
            type: 'switch',
            content: editor.language.get(CodeBlcok.cardName, 'autoWrap'),
            getState: function getState() {
              if (_classPrivateFieldLooseBase(_this3, _viewAutoWrap)[_viewAutoWrap] === undefined) {
                var _this3$getValue;

                _classPrivateFieldLooseBase(_this3, _viewAutoWrap)[_viewAutoWrap] = !!((_this3$getValue = _this3.getValue()) === null || _this3$getValue === void 0 ? void 0 : _this3$getValue.autoWrap);
              }

              return _classPrivateFieldLooseBase(_this3, _viewAutoWrap)[_viewAutoWrap];
            },
            onClick: function onClick() {
              var _this3$codeEditor;

              var autoWrap = !_classPrivateFieldLooseBase(_this3, _viewAutoWrap)[_viewAutoWrap];
              _classPrivateFieldLooseBase(_this3, _viewAutoWrap)[_viewAutoWrap] = autoWrap;
              (_this3$codeEditor = _this3.codeEditor) === null || _this3$codeEditor === void 0 ? void 0 : _this3$codeEditor.setAutoWrap(autoWrap);
            }
          }];
        }

        return [{
          key: 'dnd',
          type: 'dnd'
        }, {
          key: 'copy',
          type: 'copy'
        }, {
          key: 'delete',
          type: 'delete'
        }, {
          key: 'select',
          type: 'node',
          node: $('<div />'),
          didMount: function didMount(node) {
            // 等待编辑插件渲染成功后才能去到mode
            setTimeout(function () {
              renderSelect(node.get(), _this3.constructor.getModes(), _classPrivateFieldLooseBase(_this3, _modeNameMap)[_modeNameMap][_this3.codeEditor.mode] || _this3.codeEditor.mode || 'plain', function (mode) {
                setTimeout(function () {
                  _this3.focusEditor(); // 解决vue自动给 this.codeEditor 包 Proxy 的问题


                  if (_this3.codeEditor) {
                    toRaw(_this3.codeEditor).update(mode);
                  }
                }, 100);
              });
            }, 100);
          }
        }, {
          key: 'autoWrap',
          type: 'switch',
          content: editor.language.get(CodeBlcok.cardName, 'autoWrap'),
          getState: function getState() {
            var _this3$getValue2;

            return !!((_this3$getValue2 = _this3.getValue()) === null || _this3$getValue2 === void 0 ? void 0 : _this3$getValue2.autoWrap);
          },
          onClick: function onClick() {
            var _this3$codeEditor2;

            var value = _this3.getValue();

            var autoWrap = !(value === null || value === void 0 ? void 0 : value.autoWrap);

            _this3.setValue({
              autoWrap: autoWrap
            });

            (_this3$codeEditor2 = _this3.codeEditor) === null || _this3$codeEditor2 === void 0 ? void 0 : _this3$codeEditor2.setAutoWrap(autoWrap);
          }
        }];
      };

      var options = (_editor$plugin$findPl = editor.plugin.findPlugin('codeblock')) === null || _editor$plugin$findPl === void 0 ? void 0 : _editor$plugin$findPl.options;

      if (options === null || options === void 0 ? void 0 : options.cardToolbars) {
        return options.cardToolbars(getItems(), this.editor);
      }

      return getItems();
    }
  }, {
    key: "focusEditor",
    value: function focusEditor() {
      var _this$codeEditor2;

      (_this$codeEditor2 = this.codeEditor) === null || _this$codeEditor2 === void 0 ? void 0 : _this$codeEditor2.focus();
      this.editor.card.activate(this.root);
    }
  }, {
    key: "onSelectLeft",
    value: function onSelectLeft(event) {
      var _this$toolbarModel;

      if (!this.codeEditor) return;
      event.preventDefault();
      this.codeEditor.select(false);
      this.activate(true);
      (_this$toolbarModel = this.toolbarModel) === null || _this$toolbarModel === void 0 ? void 0 : _this$toolbarModel.show();
    }
  }, {
    key: "onSelectRight",
    value: function onSelectRight(event) {
      var _this$toolbarModel2;

      if (!this.codeEditor) return;
      event.preventDefault();
      this.codeEditor.select(true);
      this.activate(true);
      (_this$toolbarModel2 = this.toolbarModel) === null || _this$toolbarModel2 === void 0 ? void 0 : _this$toolbarModel2.show();
    }
  }, {
    key: "onSelectDown",
    value: function onSelectDown(event) {
      var _this$toolbarModel3;

      if (!this.codeEditor) return;
      event.preventDefault();
      this.codeEditor.select(true);
      this.activate(true);
      (_this$toolbarModel3 = this.toolbarModel) === null || _this$toolbarModel3 === void 0 ? void 0 : _this$toolbarModel3.show();
    }
  }, {
    key: "onSelectUp",
    value: function onSelectUp(event) {
      var _this$toolbarModel4;

      if (!this.codeEditor) return;
      event.preventDefault();
      this.codeEditor.select(false);
      this.activate(true);
      (_this$toolbarModel4 = this.toolbarModel) === null || _this$toolbarModel4 === void 0 ? void 0 : _this$toolbarModel4.show();
    }
  }, {
    key: "render",
    value: function render() {
      if (!this.codeEditor) return;

      if (!this.codeEditor.container.inEditor()) {
        this.codeEditor.container = $(this.codeEditor.renderTemplate());
        this.mirror = undefined;
        this.getCenter().empty().append(this.codeEditor.container);
      }

      var value = this.getValue();
      var mode = (value === null || value === void 0 ? void 0 : value.mode) || 'plain';
      var code = (value === null || value === void 0 ? void 0 : value.code) || '';

      if (isEngine(this.editor)) {
        var _this$codeEditor3;

        if (this.mirror) {
          this.codeEditor.update(mode, code);
          this.codeEditor.setAutoWrap(!!(value === null || value === void 0 ? void 0 : value.autoWrap));
          return;
        }

        this.mirror = (_this$codeEditor3 = this.codeEditor) === null || _this$codeEditor3 === void 0 ? void 0 : _this$codeEditor3.create(mode, code, {
          lineWrapping: !!(value === null || value === void 0 ? void 0 : value.autoWrap)
        });
      } else {
        var _this$codeEditor4;

        (_this$codeEditor4 = this.codeEditor) === null || _this$codeEditor4 === void 0 ? void 0 : _this$codeEditor4.create(mode, code, {
          lineWrapping: !!(value === null || value === void 0 ? void 0 : value.autoWrap)
        });
      }
    }
  }], [{
    key: "cardName",
    get: function get() {
      return 'codeblock';
    }
  }, {
    key: "cardType",
    get: function get() {
      return CardType.BLOCK;
    }
  }, {
    key: "autoSelected",
    get: function get() {
      return false;
    }
  }, {
    key: "singleSelectable",
    get: function get() {
      return false;
    }
  }, {
    key: "getModes",
    value: function getModes() {
      return datas;
    }
  }, {
    key: "lazyRender",
    get: function get() {
      return true;
    }
  }]);

  return CodeBlcok;
}(Card);

var en = {
  codeblock: {
    autoWrap: 'Auto Wrap'
  }
};

var cn = {
  codeblock: {
    autoWrap: '自动换行'
  }
};

var locales = {
  'en-US': en,
  'zh-CN': cn
};

var DATA_SYNTAX = 'data-syntax';
var PARSE_HTML = 'parse:html';
var PASTE_SCHEMA = 'paste:schema';
var PASTE_EACH = 'paste:each';
var MARKDOWN_IT = 'markdown-it';

var _default = /*#__PURE__*/function (_Plugin) {
  _inherits(_default, _Plugin);

  var _super = _createSuper(_default);

  function _default() {
    var _this;

    _classCallCheck(this, _default);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));

    _this.markdownIt = function (mardown) {
      if (_this.options.markdown !== false) {
        mardown.enable('code');
        mardown.enable('fence');
      }
    };

    _this.pasteSchema = function (schema) {
      var _attributes, _attributes2, _attributes3;

      schema.add([{
        type: 'block',
        name: 'pre',
        attributes: (_attributes = {}, _defineProperty(_attributes, DATA_SYNTAX, '*'), _defineProperty(_attributes, "class", '*'), _defineProperty(_attributes, "language", '*'), _defineProperty(_attributes, 'auto-wrap', '*'), _attributes)
      }, {
        type: 'block',
        name: 'code',
        attributes: (_attributes2 = {}, _defineProperty(_attributes2, DATA_SYNTAX, {
          required: true,
          value: '*'
        }), _defineProperty(_attributes2, 'auto-wrap', '*'), _attributes2)
      }, {
        type: 'block',
        name: 'code',
        attributes: {
          language: {
            required: true,
            value: '*'
          }
        }
      }, {
        type: 'block',
        name: 'code',
        attributes: {
          class: {
            required: true,
            value: function value(val) {
              return val.includes('language');
            }
          }
        },
        allowIn: ['pre', '$root']
      }, {
        type: 'block',
        name: 'div',
        attributes: (_attributes3 = {}, _defineProperty(_attributes3, DATA_SYNTAX, {
          required: true,
          value: '*'
        }), _defineProperty(_attributes3, 'auto-wrap', '*'), _attributes3)
      }]);
    };

    _this.pasteHtml = function (node) {
      var _node$get;

      var editor = _this.editor;
      if (!isEngine(editor) || node.isText()) return;

      if (((_node$get = node.get()) === null || _node$get === void 0 ? void 0 : _node$get.hasAttribute(DATA_SYNTAX)) || node.name === 'pre') {
        var syntax = node.attributes(DATA_SYNTAX);

        if (!syntax) {
          var getSyntaxForClass = function getSyntaxForClass(node) {
            var _node$get2;

            var classList = node === null || node === void 0 ? void 0 : (_node$get2 = node.get()) === null || _node$get2 === void 0 ? void 0 : _node$get2.classList;
            if (!classList) return;

            for (var i = 0; i < classList.length; i++) {
              var className = classList.item(i);

              if (className && className.startsWith('language-')) {
                var classArray = className.split('-');
                classArray.shift();
                return classArray.join('-');
              }
            }

            return undefined;
          };

          if (node.name === 'pre') {
            syntax = node.attributes('language');

            if (!syntax) {
              syntax = getSyntaxForClass(node);
            }
          }

          var _code = node.find('code');

          if (!syntax && _code.length > 0) {
            syntax = _code.attributes(DATA_SYNTAX) || _code.attributes('language');

            if (!syntax) {
              syntax = getSyntaxForClass(_code);
            }
          }
        }

        var code = new Parser(node, editor).toText(undefined, undefined, false);
        code = unescape(code.replace(/\u200b/g, ''));

        if (code.endsWith('\n')) {
          code = code.slice(0, -1);
        }

        editor.card.replaceNode(node, 'codeblock', {
          mode: syntax || 'plain',
          code: code,
          autoWrap: node.attributes('auto-wrap') === 'true'
        });
        node.remove();
        return false;
      }

      return true;
    };

    _this.parseHtml = function (root, callback) {
      var editor = _this.editor;
      var results = [];
      var synatxMap = {};
      CodeBlcok.getModes().forEach(function (item) {
        synatxMap[item.value] = item.syntax;
      });
      var codeEditor = new CodeBlockEditor(editor, {
        synatxMap: synatxMap,
        styleMap: _this.options.styleMap
      });
      var contentClassName = 'data-codeblock-content';
      var content = codeEditor.container.find(".".concat(contentClassName));
      content.css({
        border: '1px solid #e8e8e8',
        padding: '8px'
      });
      content.addClass(VIEW_CLASS_NAME);
      content.css('background', '#f9f9f9');
      root.find("[".concat(CARD_KEY, "=\"").concat(CodeBlcok.cardName, "\"],[").concat(READY_CARD_KEY, "=\"").concat(CodeBlcok.cardName, "\"]")).each(function (cardNode) {
        var node = $(cardNode);
        var card = editor.card.find(node);
        var value = (card === null || card === void 0 ? void 0 : card.getValue()) || decodeCardValue(node.attributes(CARD_VALUE_KEY));

        if (value) {
          node.empty();
          content.empty();
          codeEditor.render(value.mode || 'plain', value.code || '');
          var newContent = content.clone(true);
          node.append(newContent);
          node.removeAttributes(CARD_KEY);
          node.removeAttributes(CARD_TYPE_KEY);
          node.removeAttributes(CARD_VALUE_KEY);
          node.attributes(DATA_SYNTAX, value.mode || 'plain');
          node.attributes('auto-wrap', value.autoWrap ? 'true' : 'false');
          newContent.removeClass(VIEW_CLASS_NAME).removeClass(contentClassName);
          var newNode = node;

          if (callback) {
            newNode = callback(node, value);
            node.replaceWith(newNode);
          }

          results.push(newNode);
        } else node.remove();
      });
      codeEditor.destroy();
      return results;
    };

    return _this;
  }

  _createClass(_default, [{
    key: "init",
    value: function init() {
      var editor = this.editor;
      editor.language.add(locales);
      editor.on(PARSE_HTML, this.parseHtml);
      editor.on(PASTE_SCHEMA, this.pasteSchema);
      editor.on(PASTE_EACH, this.pasteHtml);

      if (isEngine(editor)) {
        editor.on(MARKDOWN_IT, this.markdownIt);
      }
    }
  }, {
    key: "execute",
    value: function execute(mode, value) {
      var editor = this.editor;
      if (!isEngine(editor)) return;
      var card = editor.card;

      if (!value) {
        var data = editor.getSelectionData();
        if (data) value = data.text;
      }

      var component = card.insert(CodeBlcok.cardName, {
        mode: mode,
        code: value
      });
      setTimeout(function () {
        component.focusEditor();
      }, 200);
    }
  }, {
    key: "hotkey",
    value: function hotkey() {
      return this.options.hotkey || '';
    }
  }, {
    key: "destroy",
    value: function destroy() {
      var editor = this.editor;
      editor.off(PARSE_HTML, this.parseHtml);
      editor.off(PASTE_SCHEMA, this.pasteSchema);
      editor.off(PASTE_EACH, this.pasteHtml);

      if (isEngine(editor)) {
        editor.off(MARKDOWN_IT, this.markdownIt);
      }
    }
  }], [{
    key: "pluginName",
    get: function get() {
      return 'codeblock';
    }
  }]);

  return _default;
}(Plugin);

export default _default;
export { CodeBlcok as CodeBlockComponent };
